/*******************
VARIABLES
*******************/
var lastModified = "2018-05-07";
var lastUploaded = "2018-05-07";

var templateVersion = "2.0.24";
var dimensions;
var scrollPos = {
	x: 0,
	y: 0
};

var expansion;
var panelContainer;
var closeButtonContainer;
var landscapeCloseButton;
var portraitCloseButton;
var clickthroughButton;
var muteButton;
var unmuteButton;
var audioControls
var videoContainer;
var videoBase;
var playVideoButton;
var video;
var logoContainer;
var lightbox;
var isLandscape;
var animationEndEvent;
var videoTrackingModule;
var videoOverrideClicks = false;
var sdkData;
var adId, rnd, uid;

var videoOriginWidth = 480;
var videoOriginHeight = 270;
var userInteracted = false

/*******************
INITIALIZATION
*******************/

function checkIfAdKitReady(event) {
	try {
		if (window.localPreview) {
			window.initializeLocalPreview(); //in localPreview.js
			initializeCreative();
			return;
		}
	}
	catch (e) {}

	adkit.onReady(initializeCreative);
}

function initializeCreative(event) {

	// so messaging can work in safe frames we need to bind the events that are present in the event manager.
	EBG.pm.bind("sendCreativeId", function () {
		eventManager.apply(this, arguments);
	}, this);
	EBG.pm.bind("eventCallback", function () {
		eventManager.apply(this, arguments);
	}, this);

	//Workaround (from QB6573) for Async EB Load where Modernizr isn't properly initialized
	typeof Modernizr == "object" && (Modernizr.touch = Modernizr.touch || "ontouchstart" in window);


	initializeGlobalVariables();
	window.registerInteraction = function () {}; //overwrite rI function because it will never actually be called
	initializeVideo(video);
	initializeCloseButton();
	displayDimensions();
	addEventListeners();

	expand();
}

function initializeGlobalVariables() {
	adId = EB._adConfig.adId;
	rnd = EB._adConfig.rnd;
	uid = EB._adConfig.uid;
	

	//dimensions = document.getElementById("dimensions"); //for testing only, show the dimensions and scrollpos of the parent
	expansion = document.getElementById("expansion");
	panelContainer = document.getElementById("panel-container");
	closeButtonContainer = document.getElementById("close-button-container");
	landscapeCloseButton = document.getElementById("landscape-close-button");
	portraitCloseButton = document.getElementById("portrait-close-button");
	clickthroughButton = document.getElementById("clickthrough-button");
	muteButton = document.getElementById("mute-button");
	unmuteButton = document.getElementById("unmute-button");
	audioControls = document.getElementById("audio-button-container");
	videoContainer = document.getElementById("video-container");
	videoBase = document.getElementById("video-base");
	playVideoButton = document.getElementById("play-video-button");
	video = document.getElementById("video");
	logoContainer = document.getElementById("logo-container");
	lightbox = document.getElementById("panel-lightbox");
	
	animationEndEvent = getCSSBrowserPrefix() === "webkit" ? "webkitAnimationEnd" : "animationend";

	// some settings for mobile devices
	// iOS 8 and below when clickable object overlay video, the click won't work, in this case, we will need lower the position of the video.
	videoOverrideClicks = EB.API.os.ios && EB.API.os.ver <= 8;
	if (EB.API.os.ios) {
		video.setAttribute("poster", "");	// remove video poster for ios devices so it won't show up before video play or before show video play button
		if (EB.API.os.ver >= 10) {
			video.setAttribute("playsinline", ""); // iOS 10 new playsinline attribute enable vidoe plays in line (muted)
		}
	}

	sdkData = EB.getSDKData();
}

function initializeVideo(video) {
	hideVideoControls();
	trackVideoInteractions(video);
	updateAudioButton();
}

function initializeCloseButton() {
	var enableSDKDefaultCloseButton = EB._adConfig && EB._adConfig.hasOwnProperty("mdEnableSDKDefaultCloseButton") && EB._adConfig.mdEnableSDKDefaultCloseButton;
	EB.API.setStyle(closeButtonContainer, {
		display: "block"
	});

	if(sdkData !== null) {
		if(sdkData.SDKType === "MRAID" && !enableSDKDefaultCloseButton) {
			// set sdk to use custom close button
			EB.setExpandProperties({
				useCustomClose: true
			});
		}else{
			EB.API.setStyle(closeButtonContainer, {
				display: "none"
			});
		}
	}
}

function addEventListeners() {
	if (playVideoButton) {
		playVideoButton.addEventListener("click", onPlayClick);
	}
	video.addEventListener("play", onVideoPlay);
	video.addEventListener("ended", onVideoEnd);
	video.addEventListener("pause", onVideoPause);
	
	landscapeCloseButton.addEventListener("click", handleCloseButtonClick);
	portraitCloseButton.addEventListener("click", handleCloseButtonClick);
	clickthroughButton.addEventListener("click", handleClickthroughButtonClick);
	muteButton.addEventListener("click", handleMuteButtonClick);
	unmuteButton.addEventListener("click", handleUnmuteButtonClick);

	window.addEventListener("resize", onPageResize);
}

/*******************
EVENT HANDLERS
*******************/
function handleExpandButtonClick() {
	EB.API.setStyle(expansion, {
		display: "block"
	});
	hideVideo();
	onPageResize();
	if (EB.API.getCustomVar("mdEnableExpandCollapseAnim")) {
		drawExpansion();
	}
	else {
		expanded();
	}
}

function handleCloseButtonClick() {
	pauseVideo();
	if (EB.API.getCustomVar("mdEnableExpandCollapseAnim")) {
		drawCollapse();
	}
	else {
		collapsed();
	}
}



function handleClickthroughButtonClick() {
	pauseVideo()
	EB.clickthrough();
}
function handleMuteButtonClick(){
	userInteracted = true;
	unMuteVideo();
}
function handleUnmuteButtonClick(){
	userInteracted = true;
	muteVideo();
}

/*******************
UTILITIES
*******************/
function getAnimationDuration(object) {
	var animationDuration = getStyle(object, "animation-duration");
	return animationDuration !== null ? (animationDuration.indexOf("ms") !== -1 ? animationDuration.replace("ms", "") * 1 : animationDuration.replace("s", "") * 1000) : 0;
}

function drawExpansion() {
	panelContainer.addEventListener(animationEndEvent, expandAnimationComplete);
	setClass(panelContainer, "expand-animation");
	// animation end event does not fire in android browser, use timer for it
	if (EB.API.os.android) {
		var animationDuration = getAnimationDuration(panelContainer);
		setTimeout(expandAnimationComplete, animationDuration);
	}
}

function expandAnimationComplete() {
	panelContainer.removeEventListener(animationEndEvent, expandAnimationComplete);
	removeClass(panelContainer, "expand-animation");
	expanded();
}

function expanded() {
	setOrientation();
	playVideo();
}

function drawCollapse() {
	panelContainer.addEventListener(animationEndEvent, collapseAnimationComplete);
	setClass(panelContainer, "collapse-animation");
	// animation end event does not fire in android browser, use timer for it
	if (EB.API.os.android) {
		var animationDuration = getAnimationDuration(panelContainer);
		setTimeout(collapseAnimationComplete, animationDuration);
	}	
}

function collapseAnimationComplete() {
	panelContainer.removeEventListener(animationEndEvent, collapseAnimationComplete);
	removeClass(panelContainer, "collapse-animation");
	collapsed();
}

function collapsed() {
	EB.API.setStyle(expansion, {
		display: "none"
	});
	EB.collapse();
}

function onPageResize() {
	setOrientation();
	setTimeout(setOrientation, 100);
}

function setOrientation() {
	var viewport = EB.API.getViewPortMetrics();
	isLandscape = viewport.width > viewport.height;
	setCloseButton();
	clipVideo();
}

function clipVideo() {
	var viewport = EB.API.getViewPortMetrics();
	var width = viewport.width;
	var height = viewport.height;
	var videoTop = videoOverrideClicks ? parseInt(getStyle(closeButtonContainer, "height")) : 0;
	var videoWidth = Math.ceil(Math.max(width, Math.max(video.videoWidth, videoOriginWidth) * (height / Math.max(video.videoHeight, videoOriginHeight))));
	var videoHeight = Math.ceil(Math.max(height, Math.max(video.videoHeight, videoOriginHeight) * (width / Math.max(video.videoWidth, videoOriginWidth))));
	EB.API.setStyle(expansion, {
		width: EBG.px(width),
		height: EBG.px(height)
	});
	EB.API.setStyle(panelContainer, {
		width: EBG.px(width),
		height: EBG.px(height)
	});
	EB.API.setStyle(videoContainer, {
		width: EBG.px(width),
		height: EBG.px(height - videoTop),
		top: EBG.px(videoTop)
	});
	EB.API.setStyle(playVideoButton, {
		backgroundSize: isLandscape ? "70% auto" : "auto 70%"
	});
	
	if (EB.API.getCustomVar("mdCropOrientation") === "both" || (EB.API.getCustomVar("mdCropOrientation") === "landscape" && isLandscape) || (EB.API.getCustomVar("mdCropOrientation") === "portrait" && !isLandscape)) {
		// windows surface seems unable to display video if video width exceeds 2730px
		if (EB.API.os.windows && EB.API.os.ua.match(/touch/i) !== null &&  EB.API.os.ua.match(/tablet/i) !== null) {
			videoWidth = Math.min(videoWidth, 2730);
		}
		EB.API.setStyle(videoBase, {
			width: EBG.px(videoWidth),
			height: EBG.px(videoHeight),
			top: EBG.px(Math.floor((height - videoHeight) / 2)),
			left: EBG.px(Math.floor((width - videoWidth) / 2))
		});
	}
	else {
		EB.API.setStyle(videoBase, {
			top: "0px",
			left: "0px",
			width: EBG.px(width),
			height: EBG.px(height)
		});
	}
}

function setCloseButton() {
	var viewport = EB.API.getViewPortMetrics();
	var isSmallScreen = parseInt(getStyle(closeButtonContainer, "width")) > viewport.width;
		
	EB.API.setStyle(portraitCloseButton, {
		display: isLandscape || isSmallScreen ? "none" : "block"
	});
	EB.API.setStyle(landscapeCloseButton, {
		display: isLandscape && !isSmallScreen ? "block" : "none"
	});
	EB.API.setStyle(closeButtonContainer, {
		opacity: 1
	});	
}

function trackVideoInteractions(video) {
	videoTrackingModule = new EBG.VideoModule(video);
}

function removeVideoListener() {
	if (playVideoButton) {
		playVideoButton.removeEventListener("click", onPlayClick);
	}
	video.removeEventListener("play", onVideoPlay);	
	video.removeEventListener("ended", onVideoEnd);
	video.removeEventListener("pause", onVideoPause);	
}

function showPlayButton() {
	EB.API.setStyle(playVideoButton, {
		display: "block"
	});
}

function hidePlayButton() {
	EB.API.setStyle(playVideoButton, {
		display: "none"
	});
}

function onVideoPlay() {
	showAudioControls()
	hidePlayButton();
	showVideo();
	if (EB.API.os.android) {
		setTimeout(clipVideo, 500);
	}
}

function onVideoEnd() {
	hideAudioControls();
	try {
		video.pause();
		video.currentTime = 0.1;
	}
	catch (e) {}
	onVideoPause();
}

function onVideoPause() {
	hideVideo();
	showPlayButton();
}

function onPlayClick() {
	EB.userActionCounter("Video_OverlayPlay_Clicked");
	try {
		if (video.muted && !userInteracted) {
			video.currentTime = 0.1;
		}
	}
	catch (e) {}
	setOrientation()
	videoTrackingModule.playVideo(true);	// user initiated
	if (EB.API.os.android) {
		onVideoPlay();
	}
	if (!userInteracted) {
		unMuteVideo();
	}
	userInteracted = true;
}

function playVideo() {
	videoTrackingModule.playVideo(false);	// try to auto play here
	if (EB.API.os.mobile) {
		setTimeout(function() {
			if (video.currentTime <= 0.1) {	// the device may not auto play if video time is not moving up, will display play button for user to click to play
				showPlayButton();
			} else {
				showVideo();
			}
		}, 1500);
	} else {
		showVideo();
	}
}

function pauseVideo() {
	try {
		video.pause();
	}
	catch (e) {}
	onVideoPause();
}

function showVideo() {
	if (videoOverrideClicks) {
		EB.API.setStyle(video, {width: "100%", height: "100%"});		
	} else {
		EB.API.setStyle(video, {opacity: 1});
	}
}

function hideVideo() {
	if (videoOverrideClicks) {	
		EB.API.setStyle(video, {width: "0px", height: "0px"});
	} else {
		EB.API.setStyle(video, {opacity: 0});
	}
}

function hideVideoControls() {
	video.removeAttribute("controls");
}

function showVideoControls() {
	video.setAttribute("controls", "");
}

function muteVideo() {
	video.muted = true;
	updateAudioButton()
}

function unMuteVideo() {
	video.muted = false;
	updateAudioButton()
}
function updateAudioButton(){
	if(video.muted){
		EB.API.setStyle(muteButton, {display: "block"});
		EB.API.setStyle(unmuteButton, {display: "none"});
	}else{
		EB.API.setStyle(muteButton, {display: "none"});
		EB.API.setStyle(unmuteButton, {display: "block"});
	}
}

function hideAudioControls() {
	EB.API.setStyle(audioControls, {display: "none"});
}

function showAudioControls() {
	EB.API.setStyle(audioControls, {display: "block"});
}

function getCSSBrowserPrefix(tester) {
	var testElem = document.createElement("div");
	var prefixes = {
		"WC": "",
		"O": "o",
		"MS": "ms",
		"Moz": "Moz",
		"Webkit": "webkit"
	};
	for (var prefix in prefixes) {
		if (testElem.style[prefixes[prefix] + "Transition"] !== undefined) {
			return prefixes[prefix];
		}
	}
	return null;
}

function getStyle(elem, styleName) {
	if (elem.currentStyle) {
		return elem.currentStyle[styleName];
	}
	else if (window.getComputedStyle) {
		return document.defaultView.getComputedStyle(elem, null).getPropertyValue(styleName);
	}
	else {
		return elem.style[styleName];
	}
}

function setClass(elem, class_name, override) {
	EB.API.setClass(elem, class_name, override);
}

function removeClass(elem, class_name) {
	EB.API.removeClass(elem, class_name);
}


/*******************
UTILITIES
*******************/

function displayDimensions() {
	if(dimensions) {
		var iw = window.innerWidth,
			ih = window.innerHeight,
			sx = scrollPos.x,
			sy = scrollPos.y;
		var str = "<br>Viewport: " + iw + "x" + ih + " | <span style='display:block'>Scroll (X: " + sx + "%; Y: " + sy + "%)</span>";
		dimensions.innerHTML = str;
	}
}

function expand() {
	handleExpandButtonClick();
}

function collapse() {
	handleCloseButtonClick();
}


/*********************************
HTML5 Event System - Do Not Modify
*********************************/
var listenerQueue;
var creativeIFrameId;

function sendMessage(type, data) {

	//note: the message type we're sending is also the name of the function inside
	//		the custom script's messageHandlers object, so the case must match.

	if(!data.type) data.type = type;
	EB._sendMessage(type, data);
}

function addCustomScriptEventListener(eventName, callback, interAd) {
	listenerQueue = listenerQueue || {};
	var data = {
		uid: uid,
		listenerId: Math.ceil(Math.random() * 1000000000),
		eventName: eventName,
		interAd: !!(interAd),
		creativeIFrameId: creativeIFrameId
	};
	sendMessage("addCustomScriptEventListener", data);
	data.callback = callback;
	listenerQueue[data.listenerId] = data;
	return data.listenerId;
}

function dispatchCustomScriptEvent(eventName, params) {
	params = params || {};
	params.uid = uid;
	params.eventName = eventName;
	params.creativeIFrameId = creativeIFrameId;
	sendMessage("dispatchCustomScriptEvent", params);
}

function removeCustomScriptEventListener(listenerId) {
	var params = {
		uid: uid,
		listenerId: listenerId,
		creativeIFrameId: creativeIFrameId
	};

	sendMessage("removeCustomScriptEventListener", params);
	if(listenerQueue[listenerId])
		delete listenerQueue[listenerId];
}

function eventManager(event) {

	var msg;
	if(typeof event == "object" && event.data) {
		msg = JSON.parse(event.data);

	}
	else {
		// this is safe frame.
		msg = {
			type: event.type,
			data: event
		};
	}
	if(msg.type && msg.data && (!uid || (msg.data.uid && msg.data.uid == uid))) {
		switch(msg.type) {
		case "sendCreativeId":
			creativeIFrameId = msg.data.creativeIFrameId;
			addCustomScriptEventListener('pageScroll', onPageScroll);
			sendMessage("dispatchScrollPos", {
				uid: uid
			});
			if(creativeContainerReady)
				creativeContainerReady();
			break;
		case "eventCallback": // Handle Callback
			var list = msg.data.listenerIds;
			var length = list.length;
			for(var i = 0; i < length; i++) {
				try {
					var t = listenerQueue[list[i]];
					if(!t) continue;
					t.callback(msg.data);
				}
				catch(e) {}
			}
			break;
		}
	}
}

window.addEventListener("message", function () {
	try {
		eventManager.apply(this, arguments);
	}
	catch(e) {}
}, false);
/*************************************
End HTML5 Event System - Do Not Modify
*************************************/

window.addEventListener("load", checkIfAdKitReady);